@@ -114,17 +114,27 @@ def main():
 
    # **** Main counting loop ****
 
-   for a in readfile:
-      if isAlnmntFile:
-	 r = a.read
-      else:
-	 r = a
-      if twoColumns and (isAlnmntFile and a.aligned):
-	 r.add_bases_to_count_array( base_arr_A )
-	 r.add_qual_to_count_array( qual_arr_A )
-      else:
-	 r.add_bases_to_count_array( base_arr_U )
-	 r.add_qual_to_count_array( qual_arr_U )   
+   i = 0
+   try:
+      for a in readfile:
+	 if isAlnmntFile:
+	    r = a.read
+	 else:
+	    r = a
+	 if twoColumns and (isAlnmntFile and a.aligned):
+	    r.add_bases_to_count_array( base_arr_A )
+	    r.add_qual_to_count_array( qual_arr_A )
+	 else:
+	    r.add_bases_to_count_array( base_arr_U )
+	    r.add_qual_to_count_array( qual_arr_U )   
+	 i += 1
+	 if i % 200000 == 0:
+            print i, "reads processed"
+   except:
+      sys.stderr.write( "Error occured in: %s\n" %
+         readfile.get_line_number_string() )
+      raise
+   print i, "reads processed"
 
 
    # **** Normalize result ****
@@ -176,12 +186,12 @@ def main():
       pyplot.subplot( 221 )
       plot_bases( base_arr_U_n )
       pyplot.ylabel( "proportion of base" )
-      pyplot.title( "non-aligned reads\n%.0f%% (%.3f mio)" % 
+      pyplot.title( "non-aligned reads\n%.0f%% (%.3f million)" % 
 	 ( 100. * nreads_U / (nreads_U+nreads_A), nreads_U / 1e6 ) )
 
       pyplot.subplot( 222 )
       plot_bases( base_arr_A_n )
-      pyplot.title( "aligned reads\n%.0f%% (%.3f mio)" % 
+      pyplot.title( "aligned reads\n%.0f%% (%.3f million)" % 
 	 ( 100. * nreads_A / (nreads_U+nreads_A), nreads_A / 1e6 ) )
 
       pyplot.subplot( 223 )
@@ -202,7 +212,7 @@ def main():
       pyplot.subplot( 211 )
       plot_bases( base_arr_U_n )
       pyplot.ylabel( "proportion of base" )
-      pyplot.title( "%.3f mio reads" % ( nreads_U / 1e6 ) )
+      pyplot.title( "%.3f million reads" % ( nreads_U / 1e6 ) )
 
       pyplot.subplot( 212 )
       pyplot.pcolor( qual_arr_U_n.T ** gamma, cmap=pyplot.cm.Greens,
